#!/bin/bash
FAIL=0

make clean
make
if [ $? -ne 0 ]; then
  echo "**** Make (compilation) FAILED"
  FAIL=1
fi

runtest() {
  NAME=$1
  EXPRETVAL=$2
  ARGS=$3
  EXPFILE=$4
  AOUTFILE=$5
  LOCALFAIL=0
  rm -f $AOUTFILE
  ./binary_dump $ARGS > $AOUTFILE 2>&1
  RETVAL=$?

  if [ $RETVAL -ne $EXPRETVAL ]; then
    echo "Test $NAME FAILED - RETVAL wrong. Expected: $EXPRETVAL Actual: $RETVAL"
    FAIL=1
    LOCALFAIL=1
  fi
  DIFFDATA=`diff -q $EXPFILE $AOUTFILE`
  if [ $? -ne 0 ]; then
    echo "Test $NAME FAILED - diff did not match: $DIFFDATA"
    FAIL=1
    LOCALFAIL=1
  fi
  if [ $LOCALFAIL -eq 0 ]; then
    echo "Test $NAME PASS"
  fi

}

runtest binary_dump_01 1 ""                          test01_eout test01_aout # usage
runtest binary_dump_02 0 "test_allbytes.dat"         test02_eout test02_aout # hex
runtest binary_dump_03 0 "-x test_allbytes.dat"      test03_eout test03_aout # hex
runtest binary_dump_04 0 "-b test_allbytes.dat"      test04_eout test04_aout # bin
runtest binary_dump_05 0 "test_empty.dat"            test05_eout test05_aout # hex
runtest binary_dump_06 0 "-b test_empty.dat"         test06_eout test06_aout # bin
runtest binary_dump_07 0 "-x test_0xACE_random.dat"  test07_eout test07_aout # hex
runtest binary_dump_08 0 "-b test_0xACE_random.dat"  test08_eout test08_aout # bin
runtest binary_dump_09 1 "-b nonexistant.dat"        test09_eout test09_aout # err: no such file
runtest binary_dump_10 1 "-q test_allbytes.dat"      test10_eout test10_aout # bad option
runtest binary_dump_11 1 "-b -x test_allbytes.dat"   test11_eout test11_aout # multi option
runtest binary_dump_12 1 "-bx test_allbytes.dat"     test12_eout test12_aout # multi option

if [ $FAIL -eq 1 ]; then
  echo "FAILING TESTS!"
  exit 13
else 
  exit 0
fi

